@@ -3,6 +3,7 @@ from typing import Dict, Iterable, List
 from pandas import DataFrame
 from pyspark.sql import DataFrame as PySparkDataFrame
 from pyspark.sql import SparkSession
+from pyspark.sql import functions as F
 
 from seshat.data_class import DFrame, SFrame
 from seshat.data_class.base import GroupSFrame
@@ -13,6 +14,10 @@ class SPFrame(SFrame):
     frame_name = "spf"
     data: PySparkDataFrame
 
+    @property
+    def empty(self, default_key=configs.DEFAULT_SF_KEY):
+        return self.data.isEmpty()
+
     def make_group(self, default_key=configs.DEFAULT_SF_KEY):
         return GroupSFrame({default_key: self}, sframe_class=self.__class__)
 
@@ -34,6 +39,19 @@ class SPFrame(SFrame):
 
     def extend_vertically(self, other: PySparkDataFrame):
         super().extend_vertically(other)
+        # Get all columns from both DataFrames
+        self_cols = set(self.data.columns)
+        other_cols = set(other.columns)
+
+        # Add missing columns to self.data
+        for col in other_cols - self_cols:
+            self.data = self.data.withColumn(col, F.lit(None).cast("string"))
+
+        # Add missing columns to other
+        for col in self_cols - other_cols:
+            other = other.withColumn(col, F.lit(None).cast("string"))
+
+        # Ensure column order matches
         self.data = self.data.unionByName(other)
 
     def extend_horizontally(
