@@ -1,12 +1,10 @@
 from .base import JobMetadata, JobExecutionSchedule, ExecutionMode, JobScheduleMode
-from .generate_story import GenerateStoryCommand
 from .setup_project import SetUpProjectCommand
 from .submit_to_network import SubmitCommand
 
 __all__ = [
     "SetUpProjectCommand",
     "SubmitCommand",
-    "GenerateStoryCommand",
     "JobMetadata",
     "JobExecutionSchedule",
     "ExecutionMode",
