@@ -129,6 +129,8 @@ class TransformerScenario:
 
         # Find changes in columns
         for sf_key in in_keys.intersection(out_keys):
+            if in_ is None:
+                continue
             diff_cols = set(in_.get_columns(sf_key)).symmetric_difference(
                 set(out.get_columns(sf_key))
             )
@@ -225,7 +227,7 @@ class TransformerScenario:
         def call_wrapper(call):
             def inner(self, *args, **kwargs):
                 sf_input = kwargs.get("sf_input")
-                if sf_input is None:
+                if sf_input is None and args:
                     sf_input = args[0]
 
                 copied_sf_input = deepcopy(sf_input)
