@@ -7,6 +7,10 @@ from pyspark.sql.functions import col as pyspark_col
 
 from seshat.data_class import SFrame
 from seshat.general.exceptions import ColIdNotSpecifiedError
+from seshat.general.transformer_story.base import (
+    TransformerScenario,
+    BaseTransformerStory,
+)
 from seshat.transformer import Transformer
 
 UpdateFuncs = Literal["sum", "mean", "replace"]
@@ -82,7 +86,7 @@ class Schema(Transformer):
 
     def convert_df(self, default: DataFrame, *args, **kwargs):
         if self.exclusive:
-            default = default[[*self.selected_cols]]
+            default = default[[*self.selected_cols]].copy()
 
         for col in self.cols:
             if col.dtype and self.convert_type:
@@ -90,6 +94,7 @@ class Schema(Transformer):
                     default[col.original] = (
                         default[col.original].str.replace("Z", "").replace("z", "")
                     )
+
                 default[col.original] = default[col.original].astype(col.dtype)
             if col.to:
                 default = default.rename(columns={col.original: col.to})
@@ -128,3 +133,48 @@ class Schema(Transformer):
         if ids:
             return ids
         raise ColIdNotSpecifiedError()
+
+
+class SchemaStory(BaseTransformerStory):
+    transformer = Schema
+    use_cases = [
+        "To standardize column names across different data sources.",
+        "To enforce specific data types for columns (e.g., converting strings to numbers or dates).",
+        "To select a specific subset of columns, dropping all others.",
+        "To prepare data for downstream processing or analysis by ensuring a consistent structure.",
+    ]
+    logic_overview = (
+        "The Schema transformer modifies the structure and data types of an SFrame "
+        "based on a predefined list of column specifications (`Col` objects). "
+        "It can rename columns, convert their data types, and selectively keep "
+        "or drop columns based on the `exclusive` flag. Type conversion can be "
+        "toggled with the `convert_type` flag."
+    )
+    steps = [
+        "Initialize the transformer with a list of `Col` objects, an `exclusive` flag, "
+        "and a `convert_type` flag.",
+        "If `exclusive` is True, the SFrame is filtered to include only the columns "
+        "specified in the `cols` list.",
+        "Iterate through each `Col` object in the `cols` list:",
+        "If `col.dtype` is specified and `convert_type` is True, the column's data "
+        "type is converted to the target type. Special handling for datetime strings "
+        "is applied if the target type starts with 'datetime'.",
+        "If `col.to` is specified, the column is renamed from `col.original` to `col.to`.",
+        "The transformed SFrame, with updated column names and data types, is returned.",
+    ]
+    tags = ["schema", "single-sf-operation"]
+    interactions = [
+        (
+            "Source",
+            "Schema can be passed to the Source, which then applies it to the data after fetching.",
+        )
+    ]
+
+    def get_scenarios(self):
+        from test.transformer.schema.test_schema import SchemaDFTestCase
+
+        return TransformerScenario.from_testcase(
+            SchemaDFTestCase, transformer=self.transformer
+        )
+
+    find_outputs = False  # Because shema outputs are very different based on the input and configuration
